@@ -40,13 +40,15 @@ from pydoll.exceptions import (
     InvalidFileExtension,
     InvalidIFrame,
     InvalidScriptWithElement,
+    InvalidTabInitialization,
+    MissingScreenshotPath,
     NetworkEventsNotEnabled,
     NoDialogPresent,
     NotAnIFrame,
     PageLoadTimeout,
     WaitElementTimeout,
 )
-from pydoll.protocol.base import EmptyResponse
+from pydoll.protocol.base import EmptyResponse, Response
 from pydoll.protocol.browser.events import (
     BrowserEvent,
     DownloadProgressEvent,
@@ -89,49 +91,15 @@ class Tab(FindElementsMixin):
     Primary interface for web page automation including navigation, DOM manipulation,
     JavaScript execution, event handling, network monitoring, and specialized tasks
     like Cloudflare bypass.
-
-    This class implements a singleton pattern based on target_id to ensure
-    only one Tab instance exists per browser tab.
     """
 
-    _instances: dict[str, 'Tab'] = {}
-
-    def __new__(
-        cls,
-        browser: 'Browser',
-        connection_port: int,
-        target_id: str,
-        browser_context_id: Optional[str] = None,
-    ) -> 'Tab':
-        """
-        Create or return existing Tab instance for the given target_id.
-
-        Args:
-            browser: Browser instance that created this tab.
-            connection_port: CDP WebSocket port.
-            target_id: CDP target identifier for this tab.
-            browser_context_id: Optional browser context ID.
-
-        Returns:
-            Tab instance (new or existing) for the target_id.
-        """
-        if target_id in cls._instances:
-            existing_instance = cls._instances[target_id]
-            existing_instance._browser = browser
-            existing_instance._connection_port = connection_port
-            existing_instance._browser_context_id = browser_context_id
-            return existing_instance
-
-        instance = super().__new__(cls)
-        cls._instances[target_id] = instance
-        return instance
-
     def __init__(
         self,
         browser: 'Browser',
-        connection_port: int,
-        target_id: str,
+        connection_port: Optional[int] = None,
+        target_id: Optional[str] = None,
         browser_context_id: Optional[str] = None,
+        ws_address: Optional[str] = None,
     ):
         """
         Initialize tab controller for existing browser tab.
@@ -141,60 +109,26 @@ class Tab(FindElementsMixin):
             connection_port: CDP WebSocket port.
             target_id: CDP target identifier for this tab.
             browser_context_id: Optional browser context ID.
+            ws_address: Optional WebSocket address for this tab.
         """
-        if hasattr(self, '_initialized') and self._initialized:
-            return
+        if not any([connection_port, target_id, ws_address]):
+            raise InvalidTabInitialization()
 
-        self._browser: 'Browser' = browser
-        self._connection_port: int = connection_port
-        self._target_id: str = target_id
-        self._connection_handler: ConnectionHandler = ConnectionHandler(
-            connection_port, self._target_id
-        )
-        self._page_events_enabled: bool = False
-        self._network_events_enabled: bool = False
-        self._fetch_events_enabled: bool = False
-        self._dom_events_enabled: bool = False
-        self._runtime_events_enabled: bool = False
-        self._intercept_file_chooser_dialog_enabled: bool = False
+        self._browser = browser
+        self._connection_port = connection_port
+        self._target_id = target_id
+        self._ws_address = ws_address
+        self._browser_context_id = browser_context_id
+        self._connection_handler = self._get_connection_handler()
+        self._page_events_enabled = False
+        self._network_events_enabled = False
+        self._fetch_events_enabled = False
+        self._dom_events_enabled = False
+        self._runtime_events_enabled = False
+        self._intercept_file_chooser_dialog_enabled = False
         self._cloudflare_captcha_callback_id: Optional[int] = None
-        self._browser_context_id: Optional[str] = browser_context_id
-        self._initialized: bool = True
         self._request: Optional[Request] = None
 
-    @classmethod
-    def _remove_instance(cls, target_id: str) -> None:
-        """
-        Remove instance from registry when tab is closed.
-
-        Args:
-            target_id: Target ID to remove from registry.
-        """
-        cls._instances.pop(target_id, None)
-
-    @classmethod
-    def get_instance(cls, target_id: str) -> Optional['Tab']:
-        """
-        Get existing Tab instance for target_id if it exists.
-
-        Args:
-            target_id: Target ID to look up.
-
-        Returns:
-            Existing Tab instance or None if not found.
-        """
-        return cls._instances.get(target_id)
-
-    @classmethod
-    def get_all_instances(cls) -> dict[str, 'Tab']:
-        """
-        Get all active Tab instances.
-
-        Returns:
-            Dictionary mapping target_id to Tab instances.
-        """
-        return cls._instances.copy()
-
     @property
     def page_events_enabled(self) -> bool:
         """Whether CDP Page domain events are enabled."""
@@ -282,7 +216,7 @@ class Tab(FindElementsMixin):
         Note:
             Intercepted requests must be explicitly continued or timeout.
         """
-        response: EmptyResponse = await self._execute_command(
+        response: Response[EmptyResponse] = await self._execute_command(
             FetchCommands.enable(
                 handle_auth_requests=handle_auth,
                 resource_type=resource_type,
@@ -392,7 +326,7 @@ class Tab(FindElementsMixin):
             Tab instance becomes invalid after calling this method.
         """
         result = await self._execute_command(PageCommands.close())
-        self._remove_instance(self._target_id)
+        self._browser._tabs_opened.pop(self._target_id)
         return result
 
     async def get_frame(self, frame: WebElement) -> IFrame:
@@ -422,7 +356,21 @@ class Tab(FindElementsMixin):
         if not iframe_target:
             raise IFrameNotFound('The target for the iframe was not found')
 
-        return Tab(self._browser, self._connection_port, iframe_target['targetId'])
+        target_id = iframe_target['targetId']
+        if target_id in self._browser._tabs_opened:
+            return self._browser._tabs_opened[target_id]
+
+        tab = Tab(
+            self._browser,
+            target_id=target_id,
+            connection_port=self._connection_port,
+        )
+        self._browser._tabs_opened[target_id] = tab
+        return tab
+
+    async def bring_to_front(self):
+        """Brings the page to front."""
+        return await self._execute_command(PageCommands.bring_to_front())
 
     async def get_cookies(self) -> list[Cookie]:
         """Get all cookies accessible from current page."""
@@ -545,6 +493,7 @@ class Tab(FindElementsMixin):
         self,
         path: Optional[str] = None,
         quality: int = 100,
+        beyond_viewport: bool = False,
         as_base64: bool = False,
     ) -> Optional[str]:
         """
@@ -553,6 +502,8 @@ class Tab(FindElementsMixin):
         Args:
             path: File path for screenshot (extension determines format).
             quality: Image quality 0-100 (default 100).
+            beyond_viewport: The page will be scrolled to the bottom and the screenshot will
+                include the entire page
             as_base64: Return as base64 string instead of saving file.
 
         Returns:
@@ -560,10 +511,10 @@ class Tab(FindElementsMixin):
 
         Raises:
             InvalidFileExtension: If file extension not supported.
-            ValueError: If path is None and as_base64 is False.
+            MissingScreenshotPath: If path is None and as_base64 is False.
         """
         if not path and not as_base64:
-            raise ValueError('path is required when as_base64 is False')
+            raise MissingScreenshotPath()
 
         output_extension = path.split('.')[-1] if path else ScreenshotFormat.PNG
         if not ScreenshotFormat.has_value(output_extension):
@@ -573,6 +524,7 @@ class Tab(FindElementsMixin):
             PageCommands.capture_screenshot(
                 format=ScreenshotFormat.get_value(output_extension),
                 quality=quality,
+                capture_beyond_viewport=beyond_viewport,
             )
         )
         screenshot_data = response['result']['data']
@@ -997,6 +949,11 @@ class Tab(FindElementsMixin):
         """Clear all registered event callbacks."""
         await self._connection_handler.clear_callbacks()
 
+    def _get_connection_handler(self) -> ConnectionHandler:
+        if self._ws_address:
+            return ConnectionHandler(ws_address=self._ws_address)
+        return ConnectionHandler(self._connection_port, self._target_id)
+
     async def _execute_script_with_element(self, script: str, element: WebElement):
         """
         Execute script with element context.
